;*** Serial I/O UTILITY SUBROUTINES for USART0 ***
;For AVR assembler 2
;requires macros.inc and ASCII_table.asm to assemble file
;
;Copy next 2 lines in definition file and define accordingly
;Comms settings
;#define	baud_0 9600					;Baud rate
;#define	fosc 8000000				;Clock frequency

.CSEG
;USART0_INIT - Initialize USART0
USART0_INIT:
	ldi		temp,low(fosc/(16*baud_0)-1)
	ldi		temp1,high(fosc/(16*baud_0)-1)
SETBAUD_0:
	STORE	UBRR0H,temp1			;baud register high
	STORE	UBRR0L,temp				;baud register low	
	ldi		temp,(1<<RXEN0)|(1<<TXEN0)		
	STORE	UCSR0B,temp				;enable bits
	ret     

;Inputs a character and echoes it back if char received.
ECHO_0:
	rcall   CHIN_0
	brtc	NOECHO_0
	rcall   COUT_0
NOECHO_0:
	ret

;CHIN_0 Read from USART0. Return T bit clear=nochar,
;T bit set=char received and in temp
CHIN_0:
	LOAD	temp1,UCSR0A
	sbrs	temp1,RXC0		;Check if char received
	rjmp	NOCHAR_0		;jump if no data
	LOAD	temp,UDR0		;read data
	set						;Set T flag=Char in OK
	ret
NOCHAR_0:
	clt						;Clear T flag
	ret

;COUT Output temp to UART.
COUT_0:
	SKBS	UCSR0A,UDRE0,temp2	;Check if data register empty
	rjmp	COUT_0				;loop until UDRE0=1
	STORE	UDR0,temp			;send character     
	ret

;Print a carriage return and line feed
CRLF_0:
	ldi		zl,low(CRLF_MSG_0*2)
	ldi		zh,high(CRLF_MSG_0*2)

;MSG_OUT_0 Output string of ASCII bytes in Flash 
;starting at Z until end of text EOT (EOT $04).
MSG_OUT_0:
	push	temp
MSG_OUT1_0:
	lpm		temp,z+			;read char
	cpi		temp,EOT		;Check for EOT
	breq	END_MSG_0		;Exit if EOT
	rcall	COUT_0    		;output character
	rjmp	MSG_OUT1_0
END_MSG_0:
	pop		temp
	ret

;OUTSPACE-Output a space.       
OUTSPACE_0:
	ldi		temp,' '		;output a space
	rjmp	COUT_0

;TABTO_0 - move cursor over to column # in temp1.
TABTO_0:
	push	temp
TABTOLP_0:
	push	temp1
	rcall	OUTSPACE_0
	pop		temp1
	dec		temp1
	brne	TABTOLP_0
	pop		temp
	ret

CRLF_MSG_0:
.db     CR,LF,EOT,0			;Add 0 to even number of bytes

;Converts a binary byte in temp to ASCII chars
;and prints it.

ASCII_OUT_0:
	push	temp1
	push	temp	
	swap    temp		;Print MSB first			
	rcall	DIGIT0_0	;Add appropriate offset
	rcall	cout_0		;and print
	pop		temp		;Reload char
	push	temp
	rcall	DIGIT0_0	;Add appropriate offset
	rcall	cout_0		;and print
	pop		temp
	pop		temp1	
	ret		
DIGIT0_0:
	andi	temp,0x0f	
	cpi		temp,10		;Check if 0-9
	brlo    DIGDEC_0	;Yes
	ldi		temp1,0x37	;No, A-F add 0x37
	add		temp,temp1
	ret
DIGDEC_0:
	ldi		temp1,0x30	;0-9 add 0x30
	add		temp,temp1        
	ret

;MSG_IN_0:
;        ret

tx0_int_off:
	CLRB	UCSR0B,TXCIE0,temp1
	ret

tx0_int_on:
	SETB	UCSR0B,TXCIE0,temp1
	ret

rx0_int_off:
	CLRB	UCSR0B,RXCIE0,temp1
	ret

rx0_int_on:
	SETB	UCSR0B,RXCIE0,temp1
	ret
